package javax.swing.text;

import java.util.Enumeration;
import java.io.IOException;
import java.io.Writer;

public abstract class AbstractWriter
{
    private ElementIterator it;
    private Writer out;
    private int indentLevel;
    private int indentSpace;
    private Document doc;
    private int maxLineLength;
    private int currLength;
    private int startOffset;
    private int endOffset;
    private int offsetIndent;
    private String lineSeparator;
    private boolean canWrapLines;
    private boolean isLineEmpty;
    private char[] indentChars;
    private char[] tempChars;
    private char[] newlineChars;
    private Segment segment;
    protected static final char NEWLINE = '\n';
    
    protected AbstractWriter(Writer writer, Document document) {
        this(writer, document, 0, document.getLength());
    }
    
    protected AbstractWriter(Writer out, Document doc, int startOffset, int n) {
        this.indentLevel = 0;
        this.indentSpace = 2;
        this.doc = null;
        this.maxLineLength = 100;
        this.currLength = 0;
        this.startOffset = 0;
        this.endOffset = 0;
        this.offsetIndent = 0;
        this.doc = doc;
        this.it = new ElementIterator(doc.getDefaultRootElement());
        this.out = out;
        this.startOffset = startOffset;
        this.endOffset = startOffset + n;
        Object property = doc.getProperty("__EndOfLine__");
        if ((property instanceof String)) {
            this.setLineSeparator((String)property);
        }
        else {
            String property2 = null;
            try {
                property2 = System.getProperty("line.separator");
            }
            catch (SecurityException ex) {}
            if (property2 == null) {
                property2 = "\n";
            }
            this.setLineSeparator(property2);
        }
        this.canWrapLines = true;
    }
    
    protected AbstractWriter(Writer writer, Element element) {
        this(writer, element, 0, element.getEndOffset());
    }
    
    protected AbstractWriter(Writer out, Element element, int startOffset, int n) {
        this.indentLevel = 0;
        this.indentSpace = 2;
        this.doc = null;
        this.maxLineLength = 100;
        this.currLength = 0;
        this.startOffset = 0;
        this.endOffset = 0;
        this.offsetIndent = 0;
        this.doc = element.getDocument();
        this.it = new ElementIterator(element);
        this.out = out;
        this.startOffset = startOffset;
        this.endOffset = startOffset + n;
        this.canWrapLines = true;
    }
    
    public int getStartOffset() {
        return this.startOffset;
    }
    
    public int getEndOffset() {
        return this.endOffset;
    }
    
    protected ElementIterator getElementIterator() {
        return this.it;
    }
    
    protected Writer getWriter() {
        return this.out;
    }
    
    protected Document getDocument() {
        return this.doc;
    }
    
    protected boolean inRange(Element element) {
        int startOffset = this.getStartOffset();
        int endOffset = this.getEndOffset();
        return (element.getStartOffset() >= startOffset && element.getStartOffset() < endOffset) || (startOffset >= element.getStartOffset() && startOffset < element.getEndOffset());
    }
    
    protected abstract void write() throws IOException, BadLocationException;
    
    protected String getText(Element element) throws BadLocationException {
        return this.doc.getText(element.getStartOffset(), element.getEndOffset() - element.getStartOffset());
    }
    
    protected void text(Element element) throws BadLocationException, IOException {
        int max = Math.max(this.getStartOffset(), element.getStartOffset());
        int min = Math.min(this.getEndOffset(), element.getEndOffset());
        if (max < min) {
            if (this.segment == null) {
                this.segment = new Segment();
            }
            this.getDocument().getText(max, min - max, this.segment);
            if (this.segment.count > 0) {
                this.write(this.segment.array, this.segment.offset, this.segment.count);
            }
        }
    }
    
    protected void setLineLength(int maxLineLength) {
        this.maxLineLength = maxLineLength;
    }
    
    protected int getLineLength() {
        return this.maxLineLength;
    }
    
    protected void setCurrentLineLength(int currLength) {
        this.currLength = currLength;
        this.isLineEmpty = (this.currLength == 0);
    }
    
    protected int getCurrentLineLength() {
        return this.currLength;
    }
    
    protected boolean isLineEmpty() {
        return this.isLineEmpty;
    }
    
    protected void setCanWrapLines(boolean canWrapLines) {
        this.canWrapLines = canWrapLines;
    }
    
    protected boolean getCanWrapLines() {
        return this.canWrapLines;
    }
    
    protected void setIndentSpace(int indentSpace) {
        this.indentSpace = indentSpace;
    }
    
    protected int getIndentSpace() {
        return this.indentSpace;
    }
    
    public void setLineSeparator(String lineSeparator) {
        this.lineSeparator = lineSeparator;
    }
    
    public String getLineSeparator() {
        return this.lineSeparator;
    }
    
    protected void incrIndent() {
        if (this.offsetIndent > 0) {
            ++this.offsetIndent;
        }
        else if (++this.indentLevel * this.getIndentSpace() >= this.getLineLength()) {
            ++this.offsetIndent;
            --this.indentLevel;
        }
    }
    
    protected void decrIndent() {
        if (this.offsetIndent > 0) {
            --this.offsetIndent;
        }
        else {
            --this.indentLevel;
        }
    }
    
    protected int getIndentLevel() {
        return this.indentLevel;
    }
    
    protected void indent() throws IOException {
        int n = this.getIndentLevel() * this.getIndentSpace();
        if (this.indentChars == null || n > this.indentChars.length) {
            this.indentChars = new char[n];
            for (int i = 0; i < n; ++i) {
                this.indentChars[i] = ' ';
            }
        }
        int currentLineLength = this.getCurrentLineLength();
        int lineEmpty = this.isLineEmpty();
        this.output(this.indentChars, 0, n);
        if (lineEmpty && currentLineLength == 0) {
            this.isLineEmpty = true;
        }
    }
    
    protected void write(char c) throws IOException {
        if (this.tempChars == null) {
            this.tempChars = new char[128];
        }
        this.tempChars[0] = c;
        this.write(this.tempChars, 0, 1);
    }
    
    protected void write(String s) throws IOException {
        if (s == null) {
            return;
        }
        int length = s.length();
        if (this.tempChars == null || this.tempChars.length < length) {
            this.tempChars = new char[length];
        }
        s.getChars(0, length, this.tempChars, 0);
        this.write(this.tempChars, 0, length);
        return;
    }
    
    protected void writeLineSeparator() throws IOException {
        String lineSeparator = this.getLineSeparator();
        int length = lineSeparator.length();
        if (this.newlineChars == null || this.newlineChars.length < length) {
            this.newlineChars = new char[length];
        }
        lineSeparator.getChars(0, length, this.newlineChars, 0);
        this.output(this.newlineChars, 0, length);
        this.setCurrentLineLength(0);
    }
    
    protected void write(char[] array, int i, int n) throws IOException {
        if (!this.getCanWrapLines()) {
            int n7 = i + n;
            for (int l = this.indexOf(array, '\n', i, n7); l != -1; i = l + 1, l = this.indexOf(array, '\n', i, n7)) {
                if (l > i) {
                    this.output(array, i, l - i);
                }
                this.writeLineSeparator();
            }
            if (i < n7) {
                this.output(array, i, n7 - i);
            }
        }
        else {
            int n2 = i + n;
            this.getCurrentLineLength();
            int lineLength = this.getLineLength();
            while (i < n2) {
                int index = this.indexOf(array, '\n', i, n2);
                boolean n3 = false;
                boolean n4 = false;
                int currentLineLength = this.getCurrentLineLength();
                if (index != -1 && currentLineLength + (index - i) < lineLength) {
                    if (index > i) {
                        this.output(array, i, index - i);
                    }
                    i = index + 1;
                    n4 = true;
                }
                else if (index == -1 && currentLineLength + (n2 - i) < lineLength) {
                    if (n2 > i) {
                        this.output(array, i, n2 - i);
                    }
                    i = n2;
                }
                else {
                    int n5 = -1;
                    int min = Math.min(n2 - i, lineLength - currentLineLength - 1);
                    for (int j = 0; j < min; ++j) {
                        if (Character.isWhitespace(array[j + i])) {
                            n5 = j;
                        }
                    }
                    if (n5 != -1) {
                        int n6 = n5 + (i + 1);
                        this.output(array, i, n6 - i);
                        i = n6;
                        n3 = true;
                    }
                    else {
                        int k = Math.max(0, min);
                        while (k < n2 - i) {
                            if (Character.isWhitespace(array[k + i])) {
                                n5 = k;
                                break;
                            }
                            ++k;
                        }
                        if (n5 == -1) {
                            this.output(array, i, n2 - i);
                        }
                        else {
                            n2 = n5 + i;
                            if (array[n2] == '\n') {
                                this.output(array, i, n2++ - i);
                                n4 = true;
                            }
                            else {
                                this.output(array, i, ++n2 - i);
                                n3 = true;
                            }
                        }
                        i = n2;
                    }
                }
                if (n4 || n3 || i < n2) {
                    this.writeLineSeparator();
                    if (i < n2 || n4 == 0) {
                        this.indent();
                    }
                    else {
                        continue;
                    }
                }
                else {
                    continue;
                }
            }
        }
    }
    
    protected void writeAttributes(AttributeSet set) throws IOException {
        Enumeration<Object> attributeNames = set.getAttributeNames();
        while (attributeNames.hasMoreElements()) {
            Object nextElement = attributeNames.nextElement();
            this.write(" " + nextElement + "=" + set.getAttribute(nextElement));
        }
    }
    
    protected void output(char[] array, int n, int n2) throws IOException {
        this.getWriter().write(array, n, n2);
        this.setCurrentLineLength(this.getCurrentLineLength() + n2);
    }
    
    private int indexOf(char[] array, char c, int i, int n) {
        while (i < n) {
            if (array[i] == c) {
                return i;
            }
            ++i;
        }
        return -1;
    }
}
